#!/bin/bash
# vim: dict=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   lib.sh of /CoreOS/audit/Library/testing
#   Description: A set of helpers for audit testing.
#   Author: Ondrej Moris <omoris@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2019 Red Hat, Inc. All rights reserved.
#
#   This copyrighted material is made available to anyone wishing
#   to use, modify, copy, or redistribute it subject to the terms
#   and conditions of the GNU General Public License version 2.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE. See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public
#   License along with this program; if not, write to the Free
#   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
#   Boston, MA 02110-1301, USA.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   library-prefix = audit
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

true <<'=cut'
=pod

=head1 NAME

CoreOS/audit/audit - a set of helpers for audit testing

=head1 DESCRIPTION

This is a library intended for audit testing. It will set the best
setting for audit testing, can be used to configure audit and query 
audit events and related syslog messages.

=cut

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Variables
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

auditDaemonConf="/etc/audit/auditd.conf"
auditDispatcherConf=""
auditRemotePluginConf=""
auditSyslogPluginConf=""
auditLog="/var/log/audit/audit.log"

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Internal Variables
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

_auditEventSize=""

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Internal Functions
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Preparatory calculation to get the size of a single USER
# audit event.
function _auditGetEventSize() {
    if [ -z "$_auditEventSize" ]; then
        local mark=$RANDOM
        local size_before=$(stat -c "%s" $auditLog)
        local ts=$(date "+%T")
        auditctl -m "FILL-${mark}"
        for i in $(seq 1 60); do
            ausearch -ts $ts -m USER | grep "FILL-${mark}" &>/dev/null && break
            sync
            sleep 1
        done
        local size_after=$(stat -c "%s" $auditLog)    
        _auditEventSize=$[$size_after-$size_before]
    fi
    echo $_auditEventSize
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Functions
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

true <<'=cut'
=pod

=head1 FUNCTIONS

=head2 auditConfig

Configue auditd using a given key-value pair. If a third parameter
is given, then it is used as a configuration file. Otherwise 
auditd.conf is used. Return 0 in case of success.

=over

=back

=cut
function auditConfig() {
    local key=$1
    local value=$2
    local conf=${3:-"/etc/audit/auditd.conf"}
    if grep -qe "^$key " $conf; then
        sed -i "s|^$key .*|$key = $value|" $conf && return 0
    else
        echo "$key = $value" >> $conf && return 0
    fi
    
    return 1
}

true <<'=cut'
=pod

=head2 auditWaitForEvent

For a given audit event type, message and timeout in seconds, wait
at most until timeout for the message to appear in the audit log.
Fifth parameter is optional, it is a file and if is given - event
is stored in that file. Return 0 if a message was found, 1 otherwise.

=over

=back

=cut
function auditWaitForEvent() {
    local ts=$1
    local type=$2
    local msg=$3
    local timeout=$4
    local file=$5
    local out=$(mktemp) 

    local params=""
    if [ $# -gt 5 ]; then
        shift 5
        params=$@
    fi

    local found=0
    for i in $(seq 1 $timeout); do
        if [ -z "${msg}" ]; then
            ausearch -ts $ts -m $type $params >$out && found=1
        else
            ausearch -ts $ts -m $type $params | grep "${msg}" | grep "msg=" >$out && found=1
        fi
        if [ $found -eq 1 ]; then
            if [ -n "$file" ]; then
                cp $out $file
            fi
            cat $out
            rm -f $out 2>/dev/null
            return 0
        fi
        sync
        sleep 1
    done
    rlLog "Timeout for waiting for message hit and message was not found!"
    rm -f $out 2>/dev/null

    return 1
}

true <<'=cut'
=pod

=head2 auditWaitForSyslogEvent

For a given message and timeout in seconds, wait at most until timeout 
for the message to appear in the syslog log. Return 0 if a message was 
found, 1 otherwise.

=over

=back

=cut
function auditWaitForSyslogEvent() {
    local msg=$1
    local timeout=$2
    local log=${3:-"/var/log/messages"}
    local file=$4
    local out=$(mktemp) 
    local out_tmp=$(mktemp) 

    [ -z "$3" ] && ! rlIsRHEL "<8" && log="journald"

    for i in $(seq 1 $timeout); do
        if [ $log == "journald" ]; then
            journalctl -xe -nall >$out_tmp
        else
            grep -v restraint $log >$out_tmp
        fi
        if grep "${msg}" $out_tmp >$out; then
            if [ -n "$file" ]; then
                cp $out $file
            fi
            cat $out
            rm -f $out 2>/dev/null
            return 0
        fi
        sleep 1
    done

    rlLog "Timeout for waiting for message hit and message was not found!"
    rm -f $out $out_tmp 2>/dev/null

    return 1
}

true <<'=cut'
=pod

=head2 auditIncreaseLogSize

Increase size of the audit log BY given size in megabytes

=over

=back

=cut
function auditIncreaseLogSize() {
    _auditGetEventSize

    local event_size=$_auditEventSize
    local size=$1
    local timeout=${2:-}
    local events_count=$[($size*1024*1024)/$event_size]
    local initial_size=$(stat -c "%s" /var/log/audit/* | paste -sd+ | bc)

    rlLog "Audit event size is ${event_size}B"
    rlLog "Initial accumulated log size is $[initial_size/1024]kB"

    # Fill the log.
    for i in $(seq 1 $events_count); do auditctl -m "FILL"; done
    
    for i in $(seq 1 $timeout); do
        actual_size=$(stat -c "%s" /var/log/audit/* | paste -sd+ | bc)
        rlLog "Current accumulated log size is $[$actual_size/1024]kB"
        if [ $[($actual_size-$initial_size)/(1000*1000)] -ge $size ]; then
            rlLog "Current accumulated log size is $[$actual_size/1024]kB"
            return 0
        fi
        sync
        sleep 1
    done
    rlLog "Timeout for waiting for log size hit!"
    rlLog "Current log size is $[$actual_size/1024]kB ($[$initial_size/1024+($size*1020)] needed)"
    
    return 1
}

true <<'=cut'
=pod

=head2 auditLibraryLoaded

Initialization callback.

=over

=back

=cut
function auditLibraryLoaded() {
    
    auditDaemonConf="/etc/audit/auditd.conf"
    
    if ! rlIsRHEL "<8"; then
        auditDispatcherConf="/etc/audit/audisp-remote.conf"
        auditRemotePluginConf="/etc/audit/plugins.d/au-remote.conf"
        auditSyslogPluginConf="/etc/audit/plugins.d/syslog.conf"
    else
        auditDispatcherConf="/etc/audisp/audisp-remote.conf"
        auditRemotePluginConf="/etc/audisp/plugins.d/au-remote.conf"
        auditSyslogPluginConf="/etc/audisp/plugins.d/syslog.conf"
    fi
    
    return 0
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Authors
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

true <<'=cut'
=pod

=head1 AUTHORS

=over

=item *

Ondrej Moris <omoris@redhat.com>

=back

=cut
