#!/bin/bash
# vim: dict=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of /CoreOS/audit/Sanity/audit-configuration
#   Description: Test for audit configuration
#   Author: Ondrej Moris <omoris@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2010 Red Hat, Inc. All rights reserved.
#
#   This copyrighted material is made available to anyone wishing
#   to use, modify, copy, or redistribute it subject to the terms
#   and conditions of the GNU General Public License version 2.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE. See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public
#   License along with this program; if not, write to the Free
#   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
#   Boston, MA 02110-1301, USA.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include rhts environment
. /usr/share/beakerlib/beakerlib.sh || exit 1

PACKAGE="kernel"

PACKAGES="kernel audit"

COMPLETED_FLAG=/audit-configuration-done
unset REBOOT_DONE

reboot_test() {
    # Return the configuration to the default state, so that it will
    # not interefere in further tests -- I am looking at you,
    # immutable flag.
    if [ ! -e "${COMPLETED_FLAG}" ]; then
        touch "${COMPLETED_FLAG}"
        rhts-reboot
    fi
}

cleanup_pre_reboot() {
    rlPhaseStartTest "Cleanup pre-reboot"
        rlRun "service auditd stop && sleep 5" 0
        rlFileRestore
        rlRun "service auditd restart && sleep 5" 0

        rlRun "auditctl -s > final_state" 0 "Show final audit status"
        rlFileSubmit "final_state"

        rlRun "auditctl -l > final_rules" 0 "Show final audit rules"
        rlFileSubmit "final_rules"
    rlPhaseEnd
}

cleanup_post_reboot() {
    rlPhaseStartCleanup
        rlRun "auditctl -s > completed_state" 0 "Show completed audit status"
        rlFileSubmit "completed_state"

        rlRun "auditctl -l > completed_rules" 0 "Show completed audit rules"
        rlFileSubmit "completed_rules"
    rlPhaseEnd
}

actual_tests() {
    rlPhaseStartSetup
        rlAssertRpm --all || rlDie

        rlFileBackup "/etc/audit/auditd.conf"
        rlFileBackup "/var/log/audit/audit.log"
        rlFileBackup --clean /etc/audit/rules.d

        rlRun "sed -i 's/^flush.*/flush = INCREMENTAL/' /etc/audit/auditd.conf" 0
        rlRun "sed -i 's/^freq.*/freq = 1/' /etc/audit/auditd.conf" 0

        # Let's set up the default rules, while also saving the initial
        # ones, so that we can compare if something bad happens.
        rlRun "auditctl -s > initial_state" 0 "Show initial audit status"
        rlFileSubmit "initial_state"

        rlRun "auditctl -l > initial_rules" 0 "Show initial audit rules"
        rlFileSubmit "initial_rules"

        # Reset backlog_wait_time_actual in audit >= 3 to reset to the
        # default state before the test runs.
        rlRun "auditVersion=\$(rpm -q --queryformat '%{VERSION}' audit | cut -d'.' -f1)"
        if [ "${auditVersion}" -ge 3 ]; then
            rlRun "auditctl --reset_backlog_wait_time_actual" 0 "Reset backlog_wait_time_actual"
        fi

        rlRun "service auditd stop"
        rlRun "rm -f /etc/audit/rules.d/*" 0 "Removing any existing rules"
        rlRun "yum reinstall -y audit" 0 "Reinstalling audit with default configuration"
        rlRun "service auditd start && sleep 5"

        rlRun "auditctl -s > default_state" 0 "Show default audit status"
        rlFileSubmit "default_state"

        rlRun "auditctl -l > default_rules" 0 "Show default audit rules"
        rlFileSubmit "default_rules"
    rlPhaseEnd

    rlPhaseStartTest "Default setting"

        # Setup
        major=$(sed 's/[^0-9]*\([0-9]\+\)\.[0-9]\+.*/\1/' /etc/redhat-release)
        if ! rlIsRHEL "<8.2"; then
            rules_dir="/usr/share/audit/sample-rules/"
        else
            rules_dir="/usr/share/doc/audit*/rules/"
        fi
        rlRun "sed -e '/^#/d' -e '/^$/d' ${rules_dir}/10-base-config.rules | sort >expected_rules" 0 \
            "List expected base rules"

        # Select default state file based on audit version we are using.
        default_state_file=default-rhel${major}.state
        rlTestVersion "$(rpm -q --queryformat '%{VERSION}' audit)" \
            '>' 3.0.0 && default_state_file=default-rhel9.state

        rlRun "sed \"s/pid .*/pid $(pidof auditd)/\" ${default_state_file} >expected_state" 0 \
            "Setting correct pid of auditd in reference state"

        # Trigger
        rlRun "sed -e '/^#/d' -e '/^$/d' /etc/audit/audit.rules | sort >actual_rules" 0 \
            "List actual rules"
        rlRun "auditctl -s >actual_state" 0 "Show actual audit status"

        # Verification
        rlRun "diff expected_rules actual_rules" 0 "Compare that expected rules are set"
        rlRun "diff expected_state actual_state" 0 "Compare audit state to reference"
        rlFileSubmit "expected_rules" "initial_expected_rules"
        rlFileSubmit "expected_state" "initial_expected_state"

        # Clean-up
        rlRun "rm -f expected actual state" 0

    rlPhaseEnd

    rlPhaseStartTest "Adding rule"

        # Setup
        mark=$(date "+%T")
        rlRun "auditctl -D" 0

        # Trigger
        rlRun "auditctl -a always,exit -F path=/etc/shadow" 0

        # Verification
        rlRun "auditctl -l | grep '\-a always,exit -S all -F path=/etc/shadow'" 0
        rlRun "test $(auditctl -l | wc -l) -eq 1" 0
        rlRun -s "sleep 5 && ausearch -ts $mark -m CONFIG_CHANGE" 0
        rlAssertGrep "op=add_rule" $rlRun_LOG
        rlAssertGrep "res=1" $rlRun_LOG

        # Clean-up
        rlRun "auditctl -D" 0
        rlRun "rm -f $rlRun_LOG" 0

    rlPhaseEnd

    rlPhaseStartTest "Removing rule"

        # Setup
        mark=$(date "+%T")
        rlRun "auditctl -D" 0
        rlRun "auditctl -w /root" 0

        # Trigger
        rlRun "auditctl -W /root" 0

        # Verification
        rlRun "auditctl -l | grep 'No rules'" 0
        rlRun -s "sleep 5 && ausearch -ts $mark -m CONFIG_CHANGE" 0
        rlAssertGrep "op=remove_rule" $rlRun_LOG
        rlAssertGrep "res=1" $rlRun_LOG

        # Clean-up
        rlRun "auditctl -D" 0
        rlRun "rm -f $rlRun_LOG" 0

    rlPhaseEnd

    rlPhaseStartTest "Changing backlog size"

        # Setup
        mark=$(date "+%T")
        old=$(auditctl -s | grep backlog_limit | awk '{print $2}')

        # Trigger
        rlRun "auditctl -b 8192" 0

        # Verification
        rlRun "auditctl -s | grep 'backlog_limit 8192'" 0
        rlRun -s "sleep 5 && ausearch -ts $mark -m CONFIG_CHANGE" 0
        rlAssertGrep "audit_backlog_limit=8192 old=$old" $rlRun_LOG
        rlAssertGrep "res=1" $rlRun_LOG

        # Clean-up
        rlRun "auditctl -b $old" 0

    rlPhaseEnd

    rlPhaseStartTest "Setting failure flag"

        # Setup
        old=$(auditctl -s | grep failure | awk '{print $2}')

        for f in 0 1 2; do
            # Setup
            sleep 1
            mark=$(date "+%T")

            # Trigger
            rlRun "auditctl -f $f" 0

            # Verification
            rlRun "auditctl -s | grep \"failure $f\"" 0
            rlRun -s "sleep 5 && ausearch -ts $mark -m CONFIG_CHANGE" 0
            rlAssertGrep "audit_failure=$f" $rlRun_LOG
            rlAssertGrep "res=1" $rlRun_LOG
        done

        # Clean-up
        rlRun "auditctl -f $old" 0

    rlPhaseEnd

    rlPhaseStartTest "Setting rate limit"

        # Setup
        mark=$(date "+%T")
        old=$(auditctl -s | grep rate_limit | awk '{print $2}')

        # Trigger
        rlRun "auditctl -r 3" 0

        # Verification
        rlRun "auditctl -s | grep 'rate_limit 3'" 0
        rlRun -s "sleep 5 && ausearch -ts $mark -m CONFIG_CHANGE" 0
        rlAssertGrep "audit_rate_limit=3 old=$old" $rlRun_LOG
        rlAssertGrep "res=1" $rlRun_LOG

        # Clean-up
        rlRun "auditctl -b $old" 0

    rlPhaseEnd

    rlPhaseStartTest "Trimming"

        # Setup
        mark=$(date "+%T")

        # Trigger
        rlRun "auditctl -t" 0

        # Verification
        rlRun -s "sleep 5 && ausearch -ts $mark -m CONFIG_CHANGE" 0
        rlAssertGrep "op=trim" $rlRun_LOG
        rlAssertGrep "res=1" $rlRun_LOG

        # Clean-up

    rlPhaseEnd

    rlPhaseStartTest "Immutable login uid"

        # Setup
        mark=$(date "+%T")

        # Trigger
        rlRun "unbuffer auditctl --loginuid-immutable" 0
        rlRun "auditctl -s" 0

        # Verification
        rlRun -s "sleep 5 && ausearch -ts $mark -m FEATURE_CHANGE" 0
        rlAssertGrep "feature=loginuid_immutable old=0 new=1" $rlRun_LOG
        rlAssertGrep "res=1" $rlRun_LOG

        # Clean-up

    rlPhaseEnd

    rlPhaseStartTest "Setting enable flag"

        # Setup
        sleep 1
        mark=$(date "+%T")

        # Trigger
        rlRun "auditctl -e 0" 0

        # Verification
        rlRun "auditctl -s | grep 'enabled 0'" 0
        rlRun -s "sleep 5 && ausearch -ts $mark -m CONFIG_CHANGE" 0
        rlAssertGrep "audit_enabled=0" $rlRun_LOG
        rlAssertGrep "res=1" $rlRun_LOG

        # Setup
        sleep 1
        mark=$(date "+%T")

        # Trigger
        rlRun "auditctl -e 1" 0

        # Verification
        rlRun "auditctl -s | grep 'enabled 1'" 0
        rlRun -s "sleep 5 && ausearch -ts $mark -m CONFIG_CHANGE" 1

        # Setup
        sleep 1
        mark=$(date "+%T")

        # Trigger
        rlRun "auditctl -e 2" 0

        # Verification
        rlRun "auditctl -s | grep 'enabled 2'" 0
        rlRun -s "sleep 5 && ausearch -ts $mark -m CONFIG_CHANGE" 0
        rlAssertGrep "audit_enabled=2" $rlRun_LOG
        rlAssertGrep "res=1" $rlRun_LOG

        # Setup
        sleep 1
        mark=$(date "+%T")

        # Trigger
        rlRun "auditctl -e 1" 255

        # Verification
        rlRun "auditctl -s | grep 'enabled 1'" 1
        rlRun -s "sleep 5 && ausearch -ts $mark -m CONFIG_CHANGE" 0
        rlAssertGrep "audit_enabled=1" $rlRun_LOG
        rlAssertGrep "res=0" $rlRun_LOG

        # Clean-up

    rlPhaseEnd

}


rlJournalStart

if [ -e "${COMPLETED_FLAG}" ]; then
    rm -f "${COMPLETED_FLAG}"
    export REBOOT_DONE=1
fi

if [ -z "${REBOOT_DONE}" ]; then
    actual_tests
    cleanup_pre_reboot
    reboot_test
else
    cleanup_post_reboot
fi

rlJournalPrintText
rlJournalEnd
