#!/bin/bash
# vim: dict+=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of /CoreOS/audit/Sanity/ausearch-expression
#   Description: Sanity test for ausearch expression API
#   Author: Ondrej Moris <omoris@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2017 Red Hat, Inc.
#
#   This program is free software: you can redistribute it and/or
#   modify it under the terms of the GNU General Public License as
#   published by the Free Software Foundation, either version 2 of
#   the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE.  See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program. If not, see http://www.gnu.org/licenses/.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include Beaker environment
. /usr/share/beakerlib/beakerlib.sh || exit 1

PACKAGE="audit"

# Test wrapper.
function runtest() {

    # Test name and audit log file.
    local name=$1
    local log=$2

    # Search expression.
    local expr1=$3
    local op=$4
    local expr2=$5

    # Expected events.
    shift 5
    local events="$@"
    local expected_count=$(echo $events | tr ' ' '\n' | egrep "[0-9]+" | wc -l)
    
    # Search output file.
    local output=$(mktemp)
   
    # Perform search.    
    ./auparse "$log" "$expr1" "$op" "$expr2" > $output

    # Evaluate.
    count=$(egrep "[0-9]+" $output | wc -l)   

    if [ $expected_count -ne $count ]; then        
        rlFail "$name"
        rlLog "$expected_count event(s) expected, $count found"
    else
        for E in $events; do
            egrep -q "^${E}\$" $output || missing_events+="$e "
        done
        if [ "$missing_events" != "" ]; then
            rlFail "$name"
            rlLog "Missing event(s) $missing_events in search"
        else
            rlPass "$name"
        fi
    fi
    rlLog "Expression: $expr1 $op $expr2"
    rlLog "Events found: $(cat $output | tr '\n' ' ')"

    rm -f $output
}

rlJournalStart

    rlPhaseStartSetup

        rlAssertRpm "audit"
        rlRun "gcc -o auparse auparse.c -Wall -lauparse"
        rlAssertExists "auparse" || rlDie

        rlFileBackup "/etc/audit/auditd.conf"
        rlFileBackup "/var/log/audit/audit.log"

        rlServiceStop "auditd" && sleep 3

        rlRun "sed -i 's/^flush.*/flush = INCREMENTAL/' /etc/audit/auditd.conf" 0
        rlRun "sed -i 's/^freq.*/freq = 1/' /etc/audit/auditd.conf" 0

        SAMPLE_LOG="sample.log"
        ACTUAL_LOG="/var/log/audit/audit.log"

        rlRun "gcc -o auparse auparse.c -Wall -lauparse" 0 "Build auparse"
        rlRun "service auditd restart && sleep 5" 0
        
    rlPhaseEnd

    rlPhaseStartTest "Sample audit log - logical operators"
        runtest "Expression E" $SAMPLE_LOG "auid == 1000" "" "" 2
        runtest "Expression !E" $SAMPLE_LOG "!(auid == 1000)" "" "" 1 3 4
        runtest "Expression E1 && E2 (A)" $SAMPLE_LOG "auid == 1001" "&&" "uid == 1001" 4
        runtest "Expression E1 && E2 (B)" $SAMPLE_LOG "auid == 1001 && uid == 1001" "" "" 4
        runtest "Expression E1 || E2 (A)" $SAMPLE_LOG "auid == 1000" "||" "auid == 1001" 2 3 4
        runtest "Expression E1 || E2 (B)" $SAMPLE_LOG "auid == 1000 || auid == 1001" "" "" 2 3 4
        runtest "Expression !(E1 && E2)" $SAMPLE_LOG "!(auid == 1001 && uid == 1001)" "" "" 1 2 3
        runtest "Expression !(E1 || E2)" $SAMPLE_LOG "!(auid == 1000 || auid == 1001)" "" "" 1
        runtest "Expression (E1 && E2) || E3 (A)" $SAMPLE_LOG "(auid == 1001 && uid == 1001)" "||" "auid == 0" 1 4
        runtest "Expression (E1 && E2) || E3 (B)" $SAMPLE_LOG "(auid == 1001 && uid == 1001) || auid == 0" "" "" 1 4
        runtest "Expression (E1 || E2) && E3 (A)" $SAMPLE_LOG "(auid == 1001 && auid == 1000)" "||" "uid == 1001" 4
        runtest "Expression (E1 || E2) && E3 (B)" $SAMPLE_LOG "(auid == 1001 && auid == 1000) || uid == 1001" "" "" 4
    rlPhaseEnd

    rlPhaseStartTest "Sample audit log - comparison operators"
        runtest "Comparison <" $SAMPLE_LOG "auid < 1000" "" "" 1
        runtest "Comparison >" $SAMPLE_LOG "auid > 1000" "" "" 3 4
        runtest "Comparison <=" $SAMPLE_LOG "auid <= 1000" "" "" 1 2
        runtest "Comparison >=" $SAMPLE_LOG "auid >= 1000" "" "" 2 3 4
        runtest "Comparison ==" $SAMPLE_LOG "auid == 1000" "" "" 2
        runtest "Comparison !==" $SAMPLE_LOG "auid !== 1000" "" "" 1 3 4
        runtest "Comparison i=" $SAMPLE_LOG "auid i= root" "" "" 1
        runtest "Comparison i!=" $SAMPLE_LOG "auid i!= root" "" "" 2 3 4
        runtest "Comparison r=" $SAMPLE_LOG "auid r= 0" "" "" 1
        runtest "Comparison r!=" $SAMPLE_LOG "auid r!= 0" "" "" 2 3 4
    rlPhaseEnd

    rlPhaseStartTest "Sample audit log - virtual fields"
        runtest "Virtual field timestamp" $SAMPLE_LOG "\\timestamp == ts:2.0" "" "" 2 3
        runtest "Virtual field timestamp_ex" $SAMPLE_LOG "\\timestamp_ex == ts:2.0:2" "" "" 2
        runtest "Virtual field record_type (1/3)" $SAMPLE_LOG "\\record_type == 1006" "" "" 1 2 3
        runtest "Virtual field record_type (2/3)" $SAMPLE_LOG "\\record_type !== 1006" "" "" 4
        runtest "Virtual field record_type (3/3)" $SAMPLE_LOG "\\record_type == LOGIN" "" "" 1 2 3 
    rlPhaseEnd

    rlPhaseStartTest "Sample audit log - fields"
        runtest "Field auid (==)" $SAMPLE_LOG "auid == 1000" "" "" 2
        runtest "Field auid (>=)" $SAMPLE_LOG "auid >= 1000" "" "" 2 3 4
        runtest "Field uid" $SAMPLE_LOG "uid == 0" "" "" 1 2 3
        runtest "Field gid" $SAMPLE_LOG "gid == 1001" "" "" 4
    rlPhaseEnd
    
    rlPhaseStartTest "Actual audit log"
        TS_s="$(date "+%s")"
        TS_T="$(date "+%T")"
        rlRun "useradd auexpuser && sleep 5" 0
        TE_s="$(date "+%s")"
        TE_T="$(date "+%T")"
        id1=$(ausearch -r -ts $TS_T -m ADD_USER  | sed 's/.*audit([0-9.]\+:\([0-9]\+\)).*/\1/')
        id2=$(ausearch -r -ts $TS_T -m ADD_GROUP | sed 's/.*audit([0-9.]\+:\([0-9]\+\)).*/\1/')
        ausearch -ts $TS_T -te $TE_T 
        echo "id1=$id1"
        echo "id2=$id2"
        runtest "Expression for actual log" \
                $ACTUAL_LOG \
                "\\timestamp > ts:${TS_s}.0 && \\timestamp <= ts:${TE_s}.0 && \
                (\\record_type == ADD_USER || uid == 0) && !(\\record_type == USER_MGMT) && \
                !(\\record_type == SYSCALL)" \
                "" "" \
                $id1 $id2        
        rlRun "userdel -r auexpuser" 0
    rlPhaseEnd
    
    rlPhaseStartCleanup
        rlFileRestore
        rlServiceRestore "auditd"
    rlPhaseEnd

rlJournalPrintText

rlJournalEnd
