#!/bin/bash
# vim: dict+=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   test.sh of /CoreOS/audit/Sanity/bz1906065-audisp-remote-unavailable-location
#   Description: Test audisp when remote location is unavailable
#   Author: Sergio Correia <scorreia@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2022 Red Hat, Inc.
#
#   This program is free software: you can redistribute it and/or
#   modify it under the terms of the GNU General Public License as
#   published by the Free Software Foundation, either version 2 of
#   the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE.  See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program. If not, see http://www.gnu.org/licenses/.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include Beaker environment
. /usr/share/beakerlib/beakerlib.sh || exit 1

PACKAGE="audit"
PACKAGES="${PACKAGE} audispd-plugins nmap-ncat procps-ng"
AUDISP_CONF="/etc/audit/audisp-remote.conf"
AU_CONF="/etc/audit/plugins.d/au-remote.conf"
SHADOW=/etc/shadow

load_cmp() {
   awk -v n1="${1}" -v n2="${2}" 'BEGIN {printf "%s " (n1 < n2 ? "<" : ">=") " %s\n", n1, n2}'
}

check_audisp_load() {
    _thres="${1}"
    _msg="${2}"

    for _p in $(pidof audisp-remote); do
      rlRun "_load=\$(ps -p "${_p}" -o %cpu | tail -1 | awk '{ print \$1 }')"
      rlLogInfo "${_msg} CPU usage of audisp-remote (${_p}) = ${_load}"
      _eval="$(load_cmp "${_load}" "${_thres}")"
      _exp="$(printf '%s < %s' "${_load}" "${_threshold}")"
      rlAssertEquals "(pid ${_p}) CPU usage (${_load}) should be less than ${_threshold}" "${_eval}" "${_exp}"
    done
}

run_listener() {
    _port="${1}"
    _log="${2}"
    nc -lk "${_port}" >>"${_log}" 2>>"${_log}" &
    _pid="${!}"
    _ret="${?}"
    echo "${_pid}"
    return "${_ret}"
}

rlJournalStart
  rlPhaseStartSetup
    rlAssertRpm --all \
        || rlDie "cannot contine as not all required packages are installed"

    rlRun "TmpDir=\$(mktemp -d)" 0 "Creating tmp directory"
    rlRun "pushd ${TmpDir}"

    # Let's make sure there's no stale nc running.
    rlRun "pkill nc" 0-1

    # Run listener to capure the audisp logs.
    LISTENER_PORT=60
    AUDISP_LOG="${TmpDir}/audisp.log"

    rlRun "NC_PID=\$(run_listener ${LISTENER_PORT} ${AUDISP_LOG})"
    rlRun "rlFileBackup ${AUDISP_CONF} ${AU_CONF} ${SHADOW}"

    # Setup audisp and enable au-remote.
    rlRun "sed -i -e 's/^\(remote_server =\).*/\1 127.0.0.1/' ${AUDISP_CONF}"
    rlRun "sed -i -e 's/^\(format =\).*/\1 ascii/' ${AUDISP_CONF}"
    rlRun "sed -i -e 's/^\(network_failure_action =\).*/\1 syslog/' ${AUDISP_CONF}"
    rlRun "sed -i -e 's/^\(active =\).*/\1 yes/' ${AU_CONF}"

    rlServiceStart "auditd"
    rlRun "sleep 5"

    # Set up a test rule.
    rlRun "auditctl -D" 0
    rlRun "auditctl -a always,exit -F path=${SHADOW}" 0
  rlPhaseEnd

  rlPhaseStartTest "Stop remote side and check audisp-remote process load"
    _threshold='70.0'
    check_audisp_load "${_threshold}" "BEFORE stopping remote audisp: "

    rlRun "kill ${NC_PID}" 0 "Stopping remote listener process"
    printf '\n\n==================== STOPPED REMOTE LISTENER ====================\n\n' >> "${AUDISP_LOG}"

    _timeout=45
    rlRun "sleep ${_timeout}" 0 "Waiting ${_timeout} seconds"

    rlRun "echo foo >> ${SHADOW}" 0 "Adding random nonsense (foo) in ${SHADOW}"
    check_audisp_load "${_threshold}" "AFTER stopping remote audisp: "

    # As audisp remote is stopped, we should not have logged the 'echo foo'
    # call from a couple lines earlier in the audisp log.
    rlAssertNotGrep "${SHADOW}" "${AUDISP_LOG}"
  rlPhaseEnd

  rlPhaseStartTest "Check audisp-remote process reappearing"
    printf '\n\n==================== RESTARTING REMOTE LISTENER ====================\n\n' >> "${AUDISP_LOG}"

    rlRun "NC_PID=\$(run_listener ${LISTENER_PORT} ${AUDISP_LOG})"
    rlRun "echo bar >> ${SHADOW}" 0 "Adding random nonsense (bar) in ${SHADOW}"
    check_audisp_load "${_threshold}" "AFTER remote audisp reappeared: "

    _timeout=45
    rlRun "sleep ${_timeout}" 0 "Waiting ${_timeout} seconds"

    # Now that audisp remote is back, we expect to have logged the 'echo bar'
    # from a couple lines earlier in the audisp log.
    rlAssertGrep "${SHADOW}" "${AUDISP_LOG}"
  rlPhaseEnd

  rlPhaseStartCleanup
    rlFileSubmit "${AUDISP_LOG}"
    rlRun "rm -r ${TmpDir}" 0 "Removing tmp directory"
    rlRun "rlFileRestore"
    rlRun "pkill nc" 0-1 "Stopping the listener"
    rlServiceRestore "auditd"
  rlPhaseEnd

rlJournalPrintText
rlJournalEnd
# vim:set ts=2 sw=2 et:
