#!/bin/bash
# vim: dict=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of /CoreOS/audit/Sanity/options-max-log-file
#   Description: Sanity of max_log_file* options of auditd
#   Author: Ondrej Moris <omoris@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2019 Red Hat, Inc. All rights reserved.
#
#   This copyrighted material is made available to anyone wishing
#   to use, modify, copy, or redistribute it subject to the terms
#   and conditions of the GNU General Public License version 2.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE. See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public
#   License along with this program; if not, write to the Free
#   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
#   Boston, MA 02110-1301, USA.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include rhts environment
. /usr/share/beakerlib/beakerlib.sh || exit 1

PACKAGE="audit"

# Maximum log size used in this test.
MAX_LOG_SIZE=5

# Report size of given file in mega bytes.
function size_in_mb() {
    local file=$1    
    local size=$[$(stat -c "%s" $file) / (1024*1024)]
    echo $size
}

rlJournalStart

    rlPhaseStartSetup
        rlAssertRpm "$PACKAGE"
        rlRun "rlImport audit/testing" || rlDie
        rlFileBackup --clean "$auditDaemonConf"
        rlFileBackup --clean "/var/log/audit/"

        # Stop auditd.
        rlServiceStop "auditd" && sleep 3

        # Initial configuration of auditd.
        rlRun "auditConfig 'max_log_file' '$MAX_LOG_SIZE'" 0
        rlRun "auditConfig 'num_logs' '2'" 0
        
        # Start auditd.
        rlServiceStart "auditd" && sleep 3
    rlPhaseEnd

    for action in "suspend" "ignore" "syslog" "rotate" "keep_logs"; do
        rlPhaseStartTest "max_log_file_action = $action"

            # Configure action, restart auditd and store its pid.
            rlServiceStop "auditd" && sleep 3
            rlRun "auditConfig 'max_log_file_action' '$action'" 0
            rlServiceStart "auditd" && sleep 3
            pid=$(cat /var/run/auditd.pid)            
            ts=$(date "+%T")

            # Generate the first event.
            rlRun "auditctl -m 'first'" 0

            # Fill the log to exceed max_log_file size.
            rlRun "auditIncreaseLogSize $[MAX_LOG_SIZE*4/3]" 0,1
            
            # Check the action outcome.
            case $action in

                # suspend will cause the audit daemon to stop writing records to 
                # the disk.
                "suspend")
                    # Generare the last event.
                    rlRun "auditctl -m 'last'" 0

                    # Check that there is no syslog message.
                    rlRun "auditWaitForSyslogEvent 'auditd\[$pid\]: Audit daemon log file is larger than max size' 60" 1
                    rlAssertNotGrep "auditd\[$pid\]: Audit daemon log file is larger than max size" /var/log/messages

                    # Check that there is a message about suspending.
                    rlRun "auditWaitForSyslogEvent 'auditd\[$pid\]: Audit daemon is suspending logging due to logfile size' 60" 0

                    # Check that log has expected size (>= 5MB).
                    rlAssertGreaterOrEqual "audit.log should have $MAX_LOG_SIZE MB or more" "$(size_in_mb /var/log/audit/audit.log)" $MAX_LOG_SIZE

                    # Check that log rotation did not happen.
                    rlAssertNotExists "/var/log/audit/audit.log.1"

                    # Check that frist event is in the logs but the last is not
                    # because of auditd was suspended.
                    rlRun "auditWaitForEvent $ts 'USER' 'first' 60" 0
                    rlRun "auditWaitForEvent $ts 'USER' 'last' 60" 1
                     ;;
                # If set to ignore, the audit daemon does nothing. 
                "ignore")
                    # Generare the last event.
                    rlRun "auditctl -m 'last'" 0
                    
                    # Check that there is no syslog message.
                    rlAssertNotGrep "auditd\[$pid\]: Audit daemon log file is larger than max size" /var/log/messages
                    
                    # Check that log has expected size (>= 5MB).
                    rlAssertGreaterOrEqual "audit.log should have $MAX_LOG_SIZE MB or more" "$(size_in_mb /var/log/audit/audit.log)" $MAX_LOG_SIZE

                    # Check that log rotation did not happen.
                    rlAssertNotExists "/var/log/audit/audit.log.1"
                    
                    # Check that both first and last event are in the logs.
                    rlRun "auditWaitForEvent $ts 'USER' 'first' 60" 0
                    rlRun "auditWaitForEvent $ts 'USER' 'last' 60" 0
                    ;;

                # syslog means that it will issue a warning to syslog.
                "syslog")
                    # Generare the last event.
                    rlRun "auditctl -m 'last'" 0

                    # Check that there is expected syslog message.
                    rlRun "auditWaitForSyslogEvent 'auditd\[$pid\]: Audit daemon log file is larger than max size' 60" 0
                    
                    # Check that log has expected size (>= 5MB).
                    rlAssertGreaterOrEqual "audit.log should have $MAX_LOG_SIZE MB or more" "$(size_in_mb /var/log/audit/audit.log)" $MAX_LOG_SIZE

                    # Check that log rotation did not happen.
                    rlAssertNotExists "/var/log/audit/audit.log.1"

                    # Check that both first and last event are in the logs.
                    rlRun "auditWaitForEvent $ts 'USER' 'first' 60" 0
                    rlRun "auditWaitForEvent $ts 'USER' 'last' 60" 0
                    ;;

                # The keep_logs option is similar to rotate except it does not 
                # use the num_logs setting. This prevents audit logs from being 
                # overwritten. The effect is that logs accumulate and are not
                # deleted.
                "keep_logs")
                    # Fill the log to cause three log rotations.
                    rlRun "auditIncreaseLogSize $[MAX_LOG_SIZE*4/3]" 0
                    rlRun "auditIncreaseLogSize $[MAX_LOG_SIZE*4/3]" 0

                    # Generare the last event.
                    rlRun "auditctl -m 'last'" 0

                    # Check that there is no syslog message.
                    rlRun "auditWaitForSyslogEvent 'auditd\[$pid\]: Audit daemon log file is larger than max size' 60" 1

                    # Check that log rotation did happen three times (ignoring 
                    # num_logs = 2 setting).
                    rlAssertExists "/var/log/audit/audit.log"
                    rlAssertExists "/var/log/audit/audit.log.1"
                    rlAssertExists "/var/log/audit/audit.log.2"
                    rlAssertExists "/var/log/audit/audit.log.3"

                    # Check that all logs have expected size.
                    rlAssertLesser "audit.log should be smaller than $MAX_LOG_SIZE MB" "$(size_in_mb /var/log/audit/audit.log)" $MAX_LOG_SIZE
                    rlAssertEquals "audit.log.1 should have $MAX_LOG_SIZE MB" "$(size_in_mb /var/log/audit/audit.log.1)" $MAX_LOG_SIZE
                    rlAssertEquals "audit.log.2 should have $MAX_LOG_SIZE MB" "$(size_in_mb /var/log/audit/audit.log.2)" $MAX_LOG_SIZE
                    rlAssertEquals "audit.log.3 should have $MAX_LOG_SIZE MB" "$(size_in_mb /var/log/audit/audit.log.3)" $MAX_LOG_SIZE

                    # Check that both first and last event are in the logs.
                    rlRun "auditWaitForEvent $ts 'USER' 'first' 60" 0
                    rlRun "auditWaitForEvent $ts 'USER' 'last' 60" 0
                    ;;

                # The rotate option will cause the audit daemon to rotate the logs.
                "rotate")
                    # Fill the log to cause three log rotations.
                    rlRun "auditIncreaseLogSize $[MAX_LOG_SIZE*4/3] 15" 1
                    rlRun "auditIncreaseLogSize $[MAX_LOG_SIZE*4/3] 15" 1

                    # Generare the last event.
                    rlRun "auditctl -m 'last'" 0

                    # Check that there is no syslog message.
                    rlRun "auditWaitForSyslogEvent 'auditd\[$pid\]: Audit daemon log file is larger than max size' 60" 1

                    # Check that log rotation did happen three times but only
                    # one rotation (plus audit.log) is kept - to respect 
                    # num_logs = 2 setting.
                    rlAssertExists "/var/log/audit/audit.log"
                    rlAssertExists "/var/log/audit/audit.log.1"
                    rlAssertNotExists "/var/log/audit/audit.log.2"
                    rlAssertNotExists "/var/log/audit/audit.log.3"

                    # Check that all logs have expected size.
                    rlAssertLesser "audit.log should be smaller than $MAX_LOG_SIZE MB" "$(size_in_mb /var/log/audit/audit.log)" $MAX_LOG_SIZE
                    rlAssertEquals "audit.log.1 should have $MAX_LOG_SIZE MB" "$(size_in_mb /var/log/audit/audit.log.1)" $MAX_LOG_SIZE

                    # Check that last event is in the logs but first is not
                    # because of replacing rotations.
                    rlRun "auditWaitForEvent $ts 'USER' 'first' 60" 1
                    rlRun "auditWaitForEvent $ts 'USER' 'last' 60" 0
                    ;;
            esac

            # Clean-up.
            rlRun "echo ''>/var/log/audit/audit.log && sync" 0
            rlRun "rm -rf /var/log/audit/audit.log.*" 0-255
        rlPhaseEnd
    done

    rlPhaseStartTest "BZ#1672287"        
        # Configure action, num_logs options should have no effect.
        rlServiceStop "auditd" && sleep 3
        rlRun "auditConfig 'max_log_file' '$MAX_LOG_SIZE'" 0
        rlRun "auditConfig 'max_log_file_action' 'keep_logs'" 0
        rlRun "auditConfig 'num_logs' '0'" 0
        rlServiceStart "auditd" && sleep 3
        ts=$(date "+%T")

        # First event.
        rlRun "auditctl -m 'first'" 0

        # Fill the log.
        rlRun "auditIncreaseLogSize $[MAX_LOG_SIZE*4/3]" 0

        # Fill the log again and again.
        rlRun "auditIncreaseLogSize $[MAX_LOG_SIZE*4/3]" 0
        rlRun "auditIncreaseLogSize $[MAX_LOG_SIZE*4/3]" 0

        # Last event.
        rlRun "auditctl -m 'last'" 0
        
        # Check that logs were rotated correctly. Prior to fix
        # logs were not rotated when num_logs was < 2.
        rlAssertExists "/var/log/audit/audit.log"
        rlAssertExists "/var/log/audit/audit.log.1"
        rlAssertExists "/var/log/audit/audit.log.2"
        rlAssertExists "/var/log/audit/audit.log.3"

        # Both first and the last events should be in the logs.
        rlRun "auditWaitForEvent $ts 'USER' 'first' 20" 0
        rlRun "auditWaitForEvent $ts 'USER' 'last' 20" 0

        # Clean-up.
        rlRun "echo ''>/var/log/audit/audit.log && sync" 0
        rlRun "rm -rf /var/log/audit/audit.log.*" 0-255
    rlPhaseEnd

    rlPhaseStartCleanup
        rlFileRestore
        rlServiceRestore "auditd"        
    rlPhaseEnd

rlJournalPrintText

rlJournalEnd
