#!/bin/bash
# vim: dict+=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   helpers.sh of /CoreOS/audit/Sanity/account-lifecycle
#   Description: Verify user and group account lifecycle audit implementation
#   Author: Ondrej Moris <omoris@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2015 Red Hat, Inc.
#
#   This program is free software: you can redistribute it and/or
#   modify it under the terms of the GNU General Public License as
#   published by the Free Software Foundation, either version 2 of
#   the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE.  See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program. If not, see http://www.gnu.org/licenses/.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Execute command and wait for its termination, 2 seconds delay before
# execution if enforced to make sure that date_mark will be bigger than
# the previous one (date_mark is used in check_event function).
#
# $1 - command
#
# Return exit code of executed command.
#
function setpid {
    sleep 2
    date_mark="$(date +%T)"
    "$@" &
    pid=$!
    wait $pid
    return $?
}

# Generate unique user or group entry based on existing entities.
#
# $1 - user or group
#
# Return user/group name and uid/gid, die in case of error.
#
function generate_unique_entry() {
    local source=""

    if [ "$1" == "user" ]; then
        source="passwd"
    elif [ "$1" == "group" ]; then
        source="group"
    else
        rlDie "${FUNCNAME}: 'user' or 'group' as a parameter expected!"
    fi
    
    id=$(getent $source | \
                awk -F ':' '$3 < 65000 { print $3 }' | \
                sort -ug | \
                tail -1)

    [[ "$id" =~ ^[0-9]+$ ]] || rlDie "${FUNCNAME}: cannot find free id!"
    id=$[$id+1]

    name=""
    for i in `seq 1 5`; do
        attempt="${1}${RANDOM}"

        if getent $source $attempt; then
            sleep 1
            continue
        else
            name=$attempt
            break
        fi        
    done

    [ "$name" == "" ] && rlDie "${FUNCNAME}: cannot find free name!"

    echo "$name $id"
}

# Check if an unique event of given type with given message was generated.
#
# $1    - expected number of events
# $2    - message type
# $3... - message contents
#
# Return 0 if such event if found, 1 if not, 2 if the event is not unique,
# 3 if it does not contain desired message, 3 otherwise.
#
function check_event {

    local count=$1
    local type=$2
    local result=3

    [ -z "$count" ] && { \
        echo "${FUNCNAME}: Missing count parameter!";
        return $result;  }
    [ -z "$type" ] && { \
        echo "${FUNCNAME}: Missing type parameter!"
        return $result; }
    [ $# -lt 3 ] && { \
        echo "${FUNCNAME}: Missing message parameter!"
        return $result; }      

    # Wait for empty audit backlog (but 10 seconds at most).
    local counter=0
    local timeout=10
    while true; do
        [ $(auditctl -s | egrep "backlog[^_]" |
                sed 's/.*backlog[ =]\([0-9]\+\).*/\1/') -gt 0 ] || break
        echo "${FUNCNAME}: Waiting for audit backlog..."
        sleep 1
        counter=$[$counter + 1]
        [ $counter -eq $timeout ] && break
    done

    # Store event for subsequent verification.
    local event=$(mktemp)
    ausearch -if /var/log/audit/audit.log -m $type -ts $date_mark --raw > $event

    # Verify event content and possible uniqueness.
    if [ $? -eq 0 ]; then
        if [ $(cat $event | wc -l) -eq $count ]; then
            shift 2
            result=0
            while [ $# -gt 0 ]; do
                if ! egrep -q "msg='.*$1.*'" $event; then
                    echo "${FUNCNAME}: Incorrect event message ($1)"
                    cat $event
                    result=2
                    break
                fi
                shift
            done
            if [ $result -eq 0 ]; then
                echo "${FUNCNAME}: Success, correct event(s) found"
                cat $event
            fi
        else
            echo "${FUNCNAME}: Unexpected count of events"
            cat $event
            echo "ABC"
            result=2
        fi
    else
        echo "${FUNCNAME}: No event found"
        result=1
    fi

    rm -f $event

    return $result
}

# Set user or group password.
#
# $1 - user/group
# $2 - subject name (username or groupname)
# $3 - new password
#
# Return 0 if new password was set correctly, non-zero otherwise.
#
# Return 0 on success password change, non-zero on passwd error or
# missing parameters (1).
#
function set_password {

    sleep 2
    date_mark="$(date +%T)"

    local subject=$2
    local password=$3

    [ -z "$subject"  ] && { \
        echo "${FUNCNAME}: Missing subject parameter!" && return 1; }
    [ -z "$password" ] && { \
        echo "Missing password parameter!" && return 1; }

    if [ "$1" == "user" ]; then
        local tool="passwd"
    elif [ "$1" == "group" ]; then
        local tool="gpasswd"
    else
        echo "${FUNCNAME}: Missing user/group parameter!"
        return 1
    fi

    expect -c "
      spawn $tool $subject
      expect -nocase -re {new password:} {
        send \"$password\r\"
        expect -nocase -re {new password:} {
          send \"$password\r\"
          exp_wait
          exit 0
        }
      }
      exit 1"
    
    return $?
}
