#!/bin/bash
# vim: dict+=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of /CoreOS/audit/Sanity/account-lifecycle
#   Description: Verify user and group account lifecycle audit implementation
#   Author: Ondrej Moris <omoris@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2015 Red Hat, Inc.
#
#   This program is free software: you can redistribute it and/or
#   modify it under the terms of the GNU General Public License as
#   published by the Free Software Foundation, either version 2 of
#   the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE.  See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program. If not, see http://www.gnu.org/licenses/.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include Beaker environment
. /usr/share/beakerlib/beakerlib.sh || exit 1
. ./helpers.sh || exit 1

PACKAGE="audit"

rlJournalStart

    rlPhaseStartSetup
        rlAssertRpm "audit"
        rlAssertRpm "shadow-utils"
    rlPhaseEnd

    rlPhaseStartTest "Lifecycle of user and group accounts"
    
        # AUDIT_ADD_USER
        # ==============
        # a) When a user is added to the system, either manually or
        # programmatically, by adding a line to /etc/passwd, the helper
        # applications should generate 1 event using the AUDIT_ADD_USER event
        # type if this was done successfully.
        read user uid <<< "$(generate_unique_entry 'user')"
        rlRun "setpid useradd -u $uid $user" 0 "Adding new test user"

        if ! rlIsRHEL "<8.1"; then
            rlLog "Test for bz1713432"
            rlLog "Avoiding check_event function because of bad propagating of quotes in parameters."
            rlRun -s "ausearch -if /var/log/audit/audit.log -m ADD_USER -ts $date_mark --raw" 0 \
                "Search log for not contain unknown ID event message"
            rlAssertGrep "msg='.*acct=\"$user\".*'" $rlRun_LOG
            rlAssertNotGrep 'ID="unknown' $rlRun_LOG
            rm $rlRun_LOG
        else
            rlRun "check_event 1 ADD_USER op=add-user id=$uid res=success" 0 \
                  "Checking AUDIT_ADD_USER event (success)"
        fi

        # b) If there are any failures between starting the program and
        # creating the mapping, it may be logged as an AUDIT_ADD_USER event
        # with a failure result because the mapping was not created.
        rlRun "setpid useradd -u $uid $user" 1-255 "Adding existing test user"
        rlRun "check_event 1 ADD_USER op=add-user acct=\\\"$user\\\" res=failed" 0 \
              "Checking AUDIT_ADD_USER event (failure)"

        # AUDIT_ROLE_ASSIGN & AUDIT_ROLE_REMOVE (BZ#1294663 on RHEL7)
        # =====================================
        # a) If the system supports the notion of roles, such as selinux users,
        # then you would record the addition of a role to a user account with
        # an AUDIT_ROLE_ASSIGN event.    
        rlRun "setpid semanage login -a -s staff_u $user " 0 \
              "Adding selinux test user"
        rlRun "check_event 1 ROLE_ASSIGN op=login-sename,role,range \
               acct=\\\"$user\\\"" 0 \
              "Checking AUDIT_ROLE_ASSIGN event"
        
        # b) If that role were to be revoked at some point, this would be
        # recorded with an AUDIT_ROLE_REMOVE event.
        rlRun "setpid semanage login -d $user" 0 "Removing selinux test user"
        rlRun "check_event 1 ROLE_REMOVE op=login acct=\\\"$user\\\"" 0 \
              "Checking AUDIT_ROLE_REMOVE event"
        
        # AUDIT_USER_CHAUTHTOK
        # ====================
        # When a user changes their password or other secrets, like a pin, then
        # a AUDIT_USER_CHAUTHTOK event type should be used.
        passwd="User.123!"
        rlRun "setpid usermod -p $passwd $user" 0 \
              "Setting test user password (usermod)"        
        rlRun "check_event 1 USER_CHAUTHTOK op=updating-password id=$uid" 0 \
              "Checking AUDIT_USER_CHAUTHTOK event (usermod)"

        if rlIsRHEL "<7.7"; then
            rlRun "set_password user $user $passwd" 0 \
                  "Setting test user password (passwd)"                    
            rlRun "check_event 2 USER_CHAUTHTOK op=PAM:chauthtok \
                   grantors=pam_pw quality,pam_unix acct=\\\"$user\\\"" 0 \
                  "Checking AUDIT_USER_CHAUTHTOK event (passwd)"
        else
            rlRun "set_password user $user $passwd" 0 \
                  "Setting test user password (passwd)"
            rlRun "check_event 1 USER_CHAUTHTOK op=PAM:chauthtok \
                   grantors=pam_pw quality,pam_unix acct=\\\"$user\\\"" 0 \
                  "Checking AUDIT_USER_CHAUTHTOK event (passwd)"
        fi

        # AUDIT_ADD_GROUP & AUDIT_GRP_MGMT
        # ================================
        # The lifecycle of groups is similar to users.
        # a) The creation of a group id mapping by adding a line to /etc/group
        # should result in the creation of an AUDIT_ADD_GROUP event. Any events
        # generated while using the shadow group file shall result in an event
        # using the AUDIT_GRP_MGMT type because the group mapping already
        # exists.
        read group gid <<< "$(generate_unique_entry 'group')"
        rlRun "setpid groupadd -g $gid $group" 0 "Adding new test group"
        rlRun "check_event 1 ADD_GROUP op=add-group id=$gid res=success" 0 \
              "Checking AUDIT_ADD_GROUP event (success)"
        rlRun "check_event 1 GRP_MGMT op=add-shadow-group id=$gid" 0 \
              "Checking AUDIT_GRP_MGMT event (new shadow group)"
        
        # b) Any failure leading up to the addition in /etc/group should be
        # logged with the AUDIT_ADD_GROUP event type.
        rlRun "setpid groupadd -g $gid $group" 1-255 "Adding existing test group"
        rlRun "check_event 1 ADD_GROUP op=add-group acct=\\\"$group\\\" res=failed" 0 \
              "Checking AUDIT_ADD_GROUP event (failure)"

        # AUDIT_GRP_CHAUTHTOK
        # ===================
        # When a group password or other secrets, like a pin, are changed, then
        # a AUDIT_GRP_CHAUTHTOK event type should be used.
        rlRun "set_password group $group $passwd" 0 \
              "Setting test group password"
        rlRun "check_event 1 GRP_CHAUTHTOK op=change-password grp=\\\"$group\\\"" 0 \
              "Checking AUDIT_GRP_CHAUTHTOK event"

        if rlIsRHEL ">=8.1"; then
            encPass='$6$vp.Jknm6s2vSt9/I$ZLsHl.Jpxq5wn.j2JKB2J6Ff2iq4aZG/5a9T5rD5lI4/HycaP/nAZ89K5GtqMpvFGrRsYN0yXE9FDuqiiK7K3'
            rlRun "setpid groupmod -p $encPass $group" 0 \
                  "Changing test group password"
            rlRun "check_event 1 GRP_CHAUTHTOK op=changing-shadow-group acct=\\\"$group\\\"" 0 \
                  "Checking AUDIT_GRP_CHAUTHTOK event"
        fi

        rlRun "setpid gpasswd -r $group" 0 "Removing test group password"
        rlRun "check_event 1 GRP_CHAUTHTOK op=delete-group-password \
               grp=\\\"$group\\\"" 0 \
              "Checking AUDIT_GRP_CHAUTHTOK event"        
        
        # AUDIT_USER_MGMT
        # ===============
        # If the user or admin changes anything else about the account related
        # to fields in /etc/passwd or /etc/shadow, then it should be done with
        # an AUDIT_USER_MGMT event type.
        rlRun "setpid usermod -G users,$group $user" 0 \
              "Changing test user groups (setting users and test group)"        
        rlRun "check_event 4 USER_MGMT op=add-user-to-group grp=\\\"users\\\" \
               acct=\\\"$user\\\"" 0 \
              "Checking AUDIT_USER_MGMT event (users group)"
        rlRun "check_event 4 USER_MGMT op=add-user-to-group grp=\\\"$group\\\" \
               acct=\\\"$user\\\"" 0 \
              "Checking AUDIT_USER_MGMT event (test group)"
        rlRun "check_event 4 USER_MGMT op=add-user-to-shadow-group \
               grp=\\\"users\\\" acct=\\\"$user\\\"" 0 \
              "Checking AUDIT_USER_MGMT event (users shadow group)"
        rlRun "check_event 4 USER_MGMT op=add-user-to-shadow-group \
               grp=\\\"$group\\\" acct=\\\"$user\\\"" 0 \
              "Checking AUDIT_USER_MGMT event (test shadow group)"

        rlRun "setpid usermod -c test $user" 0 "Changing test user comment"
        rlRun "check_event 1 USER_MGMT op=changing-comment id=$uid" 0 \
              "Checking AUDIT_USER_MGMT event"

        rlRun "setpid usermod -e 20 $user" 0 "Changing test user exp. date"
        rlRun "check_event 1 USER_MGMT op=changing-expiration-date id=$uid" 0 \
              "Checking AUDIT_USER_MGMT event"

        # There is a difference between the number of event on RHEL6 and 7.
        rlIsRHEL "6" && c=1 || c=2
        rlRun "setpid usermod -g $gid $user" 0 \
              "Changing test user primary group"
        rlRun "check_event $c USER_MGMT op=changing-primary-group id=$uid" 0 \
              "Checking AUDIT_USER_MGMT event"
        rlRun "check_event $c USER_MGMT op=updating-home-dir-owner id=$uid" 0 \
              "Checking AUDIT_USER_MGMT event"

        rlRun "setpid usermod -d /home/${user}X $user" 0 \
              "Changing test user home directory"
        rlRun "check_event 1 USER_MGMT op=changing-home-dir id=$uid" 0 \
              "Checking AUDIT_USER_MGMT event"

        rlRun "setpid usermod -f 10 $user" 0 "Changing test user inactive days"
        rlRun "check_event 1 USER_MGMT op=changing-inactive-days id=$uid" 0 \
              "Checking AUDIT_USER_MGMT event"
        
        rlRun "setpid usermod -s /bin/true $user" 0 "Changing test user shell"
        rlRun "check_event 1 USER_MGMT op=changing-shell id=$uid" 0 \
              "Checking AUDIT_USER_MGMT event"
        
        rlRun "setpid usermod -l ${user}X $user" 0 \
              "Changing test user login name"
        rlRun "check_event 6 USER_MGMT op=changing-name id=$uid" 0 \
              "Checking AUDIT_USER_MGMT event"
        rlRun "check_event 6 USER_MGMT op=update-member-in-group \
               grp=\\\"users\\\" acct=\\\"$user\\\"" 0 \
              "Checking AUDIT_USER_MGMT event"
        rlRun "check_event 6 USER_MGMT op=update-member-in-group \
               grp=\\\"users\\\" acct=\\\"$user\\\"" 0 \
              "Checking AUDIT_USER_MGMT event"        
        rlRun "check_event 6 USER_MGMT op=update-member-in-shadow-group \
               grp=\\\"users\\\" acct=\\\"$user\\\"" 0 \
              "Checking AUDIT_USER_MGMT event"
        rlRun "check_event 6 USER_MGMT op=update-member-in-shadow-group \
               grp=\\\"$group\\\" acct=\\\"$user\\\"" 0 \
              "Checking AUDIT_USER_MGMT event"
        rlRun "check_event 6 USER_MGMT op=updating-mail-file-name id=$uid" 0 \
              "Checking AUDIT_USER_MGMT event"

        # AUDIT_DEL_USER
        # ==============
        # When the day comes that a uid mapping will be removed from
        # /etc/passwd, this should be recorded with an AUDIT_DEL_USER event.
        rlRun "groupdel $user && setpid userdel -rf ${user}X" 0 "Removing test user"
        rlRun "check_event 1 DEL_USER op=delete-user id=$uid res=success" 0 \
              "Checking AUDIT_DEL_USER event"        
        
        # AUDIT_GRP_MGMT
        # ==============
        # After the creation of the group mapping, changes to the group
        # attributes should be logged with an AUDIT_GRP_MGMT event.
        gid=$[$gid+10]
        rlRun "setpid groupmod -g $gid $group" 0 \
              "Changing test group attribute"
        rlRun "check_event 3 GRP_MGMT op=changing-group grp=\\\"$group\\\"" 0 \
              "Checking AUDIT_GRP_MGMT event (1/3)"
        rlRun "check_event 3 GRP_MGMT op=changing-group-passwd grp=\\\"$group\\\"" \
              0 "Checking AUDIT_GRP_MGMT event (2/3)"
        rlRun "check_event 3 GRP_MGMT op=modify-group acct=\\\"$group\\\"" 0 \
              "Checking AUDIT_GRP_MGMT event (3/3)"
        
        # AUDIT_DEL_GROUP & AUDIT_GRP_MGMT
        # ================================
        # When a group is revoked and the line deleted in /etc/group, then
        # an AUDIT_DEL_GROUP must be sent. If there are any failures before the
        # group mapping is removed, they should be logged under the
        # AUDIT_GRP_MGMT event type.  
        rlRun "setpid groupdel $group" 0 "Removing test group"
        rlRun "check_event 1 DEL_GROUP op=delete-group id=$gid" 0 \
              "Checking AUDIT_DEL_GROUP event"
        rlRun "check_event 1 GRP_MGMT op=delete-shadow-group id=$gid" 0 \
              "Checking AUDIT_GRP_MGMT event (shadow group removal)"
        
    rlPhaseEnd
    
    rlPhaseStartCleanup
        rlRun "rm -rf /home/$user" 0 "Clean-up"
    rlPhaseEnd

rlJournalPrintText

rlJournalEnd
