#!/bin/bash
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of /CoreOS/audit/Sanity/aulast-smoke
#   Description: Sanity test for aulast
#   Author: Ondrej Moris <omoris@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2013 Red Hat, Inc. All rights reserved.
#
#   This copyrighted material is made available to anyone wishing
#   to use, modify, copy, or redistribute it subject to the terms
#   and conditions of the GNU General Public License version 2.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE. See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public
#   License along with this program; if not, write to the Free
#   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
#   Boston, MA 02110-1301, USA.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include rhts environment
. /usr/share/beakerlib/beakerlib.sh

function get_ssh_fingerprint {
expect <<EOF
set timeout 5
spawn ssh root@localhost
expect {
  *(yes/no* { puts yes; send "yes\r"; exp_continue; }
  timeout { puts "timeout"; exit 0 }
  eof { puts "timeout"; exit 0 }
}
exit 1
EOF
return $?
}

function do_ssh {
expect <<EOF
set timeout 10
spawn ssh ${1}@localhost
expect {
  "$1@localhost's password:" { puts "${2}" ; send "${2}\r" }
  timeout { puts "timeout" ; exit 2 } 
  eof { puts "eof" ; exit 3 } 
}
expect {
  -re {.*\$} { puts "get command line"; send "whoami\r" } 
  timeout { puts "timeout" ; exit 4 } 
  eof { puts "eof" ; exit 5 } 
}
expect {
  -nocase "$1" { puts ok; send "exit\r" }
  timeout { puts "timeout" ; exit 6 } 
  eof { puts "eof" ; exit 7 } 
}
expect {
  timeout { puts "timeout" ; exit 8 } 
  eof
}  
EOF
result=$?
sleep 2
return $result
}

PACKAGE="audit"

PASS="F3E2L79458g36d7"

USER="--user"

rlIsRHEL "<7" && USER=""

rlJournalStart

rlPhaseStartSetup

  # Requirements check.
  rlCheckRpm "audit"           || rlDie
  rlCheckRpm "expect"          || rlDie
  rlCheckRpm "openssh-server"  || rlDie
  rlCheckRpm "openssh-clients" || rlDie
  rlCheckRpm "openssh"         || rlDie

  # Stop logging.
  rlServiceStop "auditd" && sleep 3
  
  # Setup auditd to log immediately.
  rlFileBackup "/etc/audit/auditd.conf"
  rlRun "sed -i 's/^flush.*/flush = INCREMENTAL/' /etc/audit/auditd.conf" 0
  rlRun "sed -i 's/^freq.*/freq = 1/' /etc/audit/auditd.conf" 0

  # Testusers creation.
  rlRun "useradd testuser" 0 "Adding testing user"
  rlRun "echo \"$PASS\" | passwd --stdin testuser" 0 "Setting password"

  rlRun "get_ssh_fingerprint" 0 "Getting SSH fingerprint"
  
  rlFileBackup "/var/log/audit/audit.log"
  rlRun "echo '' > /var/log/audit/audit.log" 0 "Cleaning audit.log"

  # Start logging.
  rlRun "service auditd start && sleep 5" 0
  
rlPhaseEnd

rlPhaseStartTest

  # Corect login.
  rlRun "do_ssh testuser $PASS" 0 "Correct login"
  rlAssertEquals "Checking correct login" \
      "`aulast $USER testuser | tr -d ' '`X" "`last | head -1 | tr -d ' '`X" 

  # A serie of correct logins.
  for I in `seq 10`; do 
      do_ssh testuser $PASS
  done
  rlAssertEquals "Checking the number of correct logins" \
      `aulast $USER testuser | wc -l` 11

  # Option --bad. Incorrect login (wrong password).
  rlRun "do_ssh testuser abcde" 8 "Incorrect login (wrong password)"
  rlRun "aulast --bad | grep testuser" 0 "Checking incorrect login"
  rlAssertEquals "Checking the number of incorrect logins" \
      `aulast --bad | wc -l` 1 
  aulast --bad
  rlAssertEquals "Checking the number of correct logins" \
      `aulast $USER testuser | wc -l` 11

  # Option --bad. Incorrect login (wrong username).
  rlRun "do_ssh testabcd abcde" 8 "Incorrect login (wrong username)"
  rlRun "aulast --bad | grep testuser" 0 "Checking incorrect login"
  rlAssertEquals "Checking the number of incorrect logins" \
      `aulast --bad | wc -l` 2 
  aulast --bad 

  # Extraction.
  rlRun "aulast --extract" 0 "Extracting records to aulast.log"
  rlAssertExists "aulast.log"

  # Option --stdin.
  rlAssertEquals "Checking '--stdin' option" \
      `ausearch -ts today | aulast $USER testuser --stdin | wc -l` 11

  # Option -f.
  do_ssh testuser $PASS
  rlAssertEquals "Checking '-f <file>' option" \
      `aulast $USER testuser -f aulast.log | wc -l` 11
  rlRun "aulast -f /non-existing-file" 1 \
      "Checking '-f' with a non-existing file"

  # Option --proof. 
  ERROR=0
  for EVENT in `aulast $USER testuser --proof | tail -4 | head -1 | \
                awk -F ':' '{print $2}' | tr -d ','`; do 
      ausearch -a $EVENT || ERROR=1
  done
  rlAssertEquals "Checking related events (single login)" $ERROR 0
  if rlIsRHEL '<6.8'; then
    : # do nothing until next audit rebase; see bz922508#18,19
  else
    ERROR=0
    aulast --proof >all.proofs
    aulast --bad --proof >>all.proofs
    for EVENT in `cat all.proofs | grep 'event' | \
                  awk -F ':' '{print $2}' | tr ',' '\n' | sort -u`; do 
        if ! ausearch -a $EVENT; then
            ERROR=1
            rlLogWarning "Event $EVENT does not exist!"
        fi
    done
    rlAssertEquals "Checking related events (all logins)" $ERROR 0
  fi

rlPhaseEnd

rlPhaseStartCleanup

  rlRun "killall -9 -u testuser" 0-255
  rlRun "userdel -rf testuser" 0 "Removing testing user"
  rlRun "rm -rf aulast.log all.proofs" 0 "Removing temporary files"

  rlFileRestore
  rlServiceRestore "auditd"

rlPhaseEnd

rlJournalEnd

rlJournalPrintText
