#!/bin/bash
# vim: dict+=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of /CoreOS/audit/Sanity/system-lifecycle
#   Description: Verify system lifecycle audit imlementation
#   Author: Ondrej Moris <omoris@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2019 Red Hat, Inc.
#
#   This program is free software: you can redistribute it and/or
#   modify it under the terms of the GNU General Public License as
#   published by the Free Software Foundation, either version 2 of
#   the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE.  See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program. If not, see http://www.gnu.org/licenses/.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include Beaker environment
. /usr/share/beakerlib/beakerlib.sh || exit 1

PACKAGE="audit"

rlJournalStart

    rlPhaseStartSetup
        rlAssertRpm "audit"
        rlImport "audit/testing"

        rlRun "service auditd restart && sleep 3" 0 "Make sure auditd is running"
    rlPhaseEnd

    if ! [ -e "/var/tmp/system-lifecycle-reboot" ]; then
        date "+%T" > "/var/tmp/system-lifecycle-reboot"
        rhts-reboot 
    fi

    rlPhaseStartTest

        # AUDIT_SYSTEM_RUNLEVEL
        # AUDIT_SYSTEM_SHUTDOWN
        # AUDIT_SERVICE_STOP
        # AUDIT_DAEMON_END
        # ================
        # When the system is to be shutdown, the init system should start by 
        # sending an AUDIT_SYSTEM_RUNLEVEL event showing current runlevel and 
        # the new level as "6" which is shutdown. Next an AUDIT_SYSTEM_SHUTDOWN 
        # event is sent to signify shutdown has started. As shutdown proceeds, 
        # init shall terminate each running daemon and record a 
        # AUDIT_SERVICE_STOP event. The final service to be shutdown should be 
        # the audit daemon. It will mark shutdown complete with a 
        # AUDIT_DAEMON_END event.
        ts=$(cat /var/tmp/system-lifecycle-reboot)
        # !!! BZ#1779636 !!!
        #rlRun "auditWaitForEvent '$ts' 'SYSTEM_RUNLEVEL' 'old-level=3 new-level=6' 30 '' '-te boot'" 0
        rlRun "auditWaitForEvent '$ts' 'SYSTEM_SHUTDOWN' '' 30 '' '-te boot'" 0
        rlRun "auditWaitForEvent '$ts' 'SERVICE_STOP' 'unit=sshd' 30 '' '-te boot'" 0
        rlRun "auditWaitForEvent '$ts' 'DAEMON_END' '' 30 '' '-te boot'" 0

        # AUDIT_SYSTEM_BOOT
        # =================
        # When the system is powered on, control is eventually turned over 
        # to an init daemon. This daemon is responsible for starting up all 
        # other system services and performing an order system shutdown when 
        # asked. This init daemon should send a AUDIT_SYSTEM_BOOT event after 
        # it has done its own initialization. It is also required that the 
        # audit daemon be started as early as possible to start pulling 
        # events out of the kernel before they are lost. Most init systems 
        # have different targets or modes of operation that the system is 
        # turned over for interactive sessions. Examples are multi-user 
        # console, multi-user graphical, etc.
        rlRun "auditWaitForEvent 'boot' 'SYSTEM_BOOT' '' 120" 0

        # AUDIT_SYSTEM_RUNLEVEL
        # =====================
        # Init will determine what runlevel the system is ultimately going 
        # to try to achieve. When gets there or it fails to get there, it shall 
        # issue an AUDIT_SYSTEM_RUNLEVEL event to denote which mode of operation
        # it was going to be in. If an admin requests that the system switch to 
        # another runlevel, then it should issue another AUDIT_SYSTEM_RUNLEVEL 
        # event. The AUDIT_SYSTEM_RUNLEVEL event must record the old level 
        # (old-level=N) and new level (new-level=5). This may be numeric or a 
        # string with no spaces depending on the init system's design.
        rlRun "auditWaitForEvent 'boot' 'SYSTEM_RUNLEVEL' 'old-level=N new-level=3'" 0
        sleep 1
        ts=$(date "+%T")
        rlRun "systemctl isolate graphical.target" 0
        rlRun "auditWaitForEvent 'boot' 'SYSTEM_RUNLEVEL' 'old-level=3 new-level=5'" 0
        rlRun "systemctl isolate multi-user.target" 0

        # AUDIT_SERVICE_START
        # ===================
        # During the process of achieving the desired runlevel, the init system
        # will start various system services or daemons. When a daemon is 
        # started, the init program shall issue a AUDIT_SERVICE_START event. The 
        # event shall record the full path to the daemon or service in a hex 
        # encoded format in a field named service. Relative paths are forbidden.
        service=$(systemctl list-units --type service --state running \
            | grep .service \
            | awk '{print $1}' \
            | awk -F '.' '{print $1}' \
            | grep -v journald)
        for S in $services; do
            rlRun "auditWaitForEvent 'boot' 'SERVICE_START' 'unit=$S'" 0
        done
    
        # AUDIT_SERVICE_START, AUDIT_DAEMON_ABORT
        # =======================================
        # One of the first daemons to start up is the audit daemon. It shall issue
        # an AUDIT_DAEMON_START to denote that the daemon has been launched. If 
        # there is an error in its configuration that will not allow it to operate,
        # it should issue an AUDIT_DAEMON_ABORT event with an error reason code.
        rlRun "auditWaitForEvent 'boot' 'DAEMON_START'" 0
        # rlRun "cp /etc/audit/auditd.conf /etc/audit/auditd.conf.bck" 0
        # rlRun "echo 'bogus' >/etc/audit/auditd.conf" 0
        # rlRun "service auditd restart" 0
        # tail -100 /var/log/messages
        # rlRun "cp /etc/audit/auditd.conf.bck /etc/audit/auditd.conf" 0
        # rlRun "service auditd restart" 0

        # AUDIT_SERVICE_STOP, AUDIT_SERVICE_START
        # =======================================
        # If an admin restarts a service to reload its configuration, the init 
        # system shall issue an AUDIT_SERVICE_STOP and AUDIT_SERVICE_START event.
        ts=$(date "+%T")
        rlRun "systemctl restart sshd" 0
        rlRun "auditWaitForEvent '$ts' 'SERVICE_STOP' 'unit=sshd'" 0
        rlRun "auditWaitForEvent '$ts' 'SERVICE_START' 'unit=sshd'" 0

    rlPhaseEnd
    
    rlPhaseStartCleanup
        rlRun "rm -f /var/tmp/system-lifecycle-*" 0
    rlPhaseEnd

rlJournalPrintText

rlJournalEnd
